SERVICEDIR="/etc/alterator/net-iptables"

regex_tu='\(tcp\|udp\)'
regex_p1='[0-9]\+\(-[0-9]\+\)\?'
regex_p2="\($regex_tu:$regex_p1\)\|\(icmp:\)"

test_portrange(){
  local prange="${1:-}"
  echo "$prange" | grep -q "^[0-9]\+\(:[0-9]\+\)\?\$" ||
    fatal "`_ "Error: bad port or port range"`: \"$prange\""
  local p1="${prange%:*}"
  local p2="${prange#*:}"
  [ "$p1" = "$prange" -o "$(($p2-$p1 > 0))" = "1" ] ||
    fatal "`_ "Error: bad port or port range"`: \"$prange\""
}

srv_expand(){
  local name
  while read name; do
    [ -n "$name" ] || continue
    if [ "$name" = "icmp:" ]; then
      srv_port2rule "$name"
    elif echo "$name" | grep -q "^$regex_p2\$"; then
      srv_port2rule "$name"
    elif echo "$name" | grep -q "^$regex_p1\$"; then
      srv_port2rule "tcp:$name"
      srv_port2rule "udp:$name"
    else
      local srv_file="$SERVICEDIR/${name##/}.desktop"
      [ -s "$srv_file" ] || fatal "`_ "Error: unknown service"` \"$name\"."
      alterator-dump-desktop -v out=X-Alterator-Port "$srv_file" |\
                   tr ';' '\n' |
      while read pp; do
        echo $pp | grep -q "^$regex_p2\$" ||\
          fatal "`_ "Error: bad port"` \"$pp\" `_ "in desktop-file for service"` \"$name\"."
        srv_port2rule "$pp"
      done
    fi
  done
}

srv_port2rule(){
  local p="${1:-}"
  if [ "$p" = "icmp:" ]; then
    echo "-p icmp"
  else
    local ports="$(echo ${p#*:} | tr '-' ':')"
    local proto="${p%${p#*:}}"
    test_portrange "$ports"
    echo "-p ${proto%:} --dport $ports"
  fi
}

srv_extract(){
  local obj="${1:-}";
  local ret sep
  while read s; do
    [ -n "$s" ] || continue
    case "$obj" in
    tcp)
      sep=','
      echo $s | grep -q "^tcp:$regex_p1\$" && ret="${ret:+$ret$sep}${s#tcp:}" ||:
      echo $s | grep -q "^$regex_p1\$"     && ret="${ret:+$ret$sep}$s" ||:
    ;;
    udp)
      sep=','
      echo $s | grep -q "^udp:$regex_p1\$" && ret="${ret:+$ret$sep}${s#udp:}" ||:
      echo $s | grep -q "^$regex_p1\$"     && ret="${ret:+$ret$sep}$s" ||:
    ;;
    srv)
      sep=';'
      if [ "$s" = "icmp:" ]; then
        ret="${ret:+$ret$sep}icmp"
      else
        echo $s | grep -q "^\($regex_p2\|$regex_p1\)\$" || ret="${ret:+$ret$sep}$s"
      fi
    ;;
    esac
  done
  echo "$ret"
}

srv_compile(){
  local srv="${1:-}"
  local tcp="${2:-}"
  local udp="${3:-}"
  local sep=';'
  local i ret
  {
    echo "$srv" | tr ';,' '\n'
    echo "$tcp" | tr ';,' '\n' |
      while read i; do [ -z "$i" ] || echo "tcp:$i"; done
    echo "$udp" | tr ';,' '\n' |
      while read i; do [ -z "$i" ] || echo "udp:$i"; done
  } | tr '\n' ';'
}

